#include "PyObjectPtr.h"
#include "PythonInterpreter.h"

#include <string>
#include <sstream>  // stringstream
#include <exception>

PyObjectPtr::PyObjectPtr(PyObject* pyobject_ptr,
                         const Status& status):
    ptr{pyobject_ptr},
    status{status}
{
    // if PyObject pointer is null but result type is not `None` or `Error`,
    // then that will be an `Error` case.
    if(!ptr && status.type != Status::Type::None
       && status.type != Status::Type::Error)
    {
        this->status.type = Status::Type::Error;
    }
}


PyObjectPtr::PyObjectPtr(PyObject* pyobject_ptr):
    PyObjectPtr(pyobject_ptr, Status(Status::Type::Info)) {}


PyObjectPtr::PyObjectPtr(const Status& status):
    PyObjectPtr(nullptr, status) {}


PyObjectPtr::~PyObjectPtr()
{
    discard();
}


PyObjectPtr::PyObjectPtr(PyObjectPtr&& other):
    PyObjectPtr(other.ptr, other.status)
{
    // reset the moved object
    other.reset();
}


PyObject* PyObjectPtr::release()
{
    PyObject* pyobject_ptr {ptr};
    reset();
    return pyobject_ptr;
}


void PyObjectPtr::reset()
{
    ptr = nullptr;
    status = Status();
}


void PyObjectPtr::discard()
{
    if (!PythonInterpreter::isInitialized()) {
        throw(std::runtime_error(
                  "Decrementing Python reference-count without "
                  "Python initialized leads to memory access violation "
                  "(segmentation fault)"));
    }

    PythonInterpreter::DecRef(ptr);
    reset();
}


bool PyObjectPtr::valid() const
{
    return (ptr && (status.isOK() || status.isWarning()));
}


bool PyObjectPtr::isSet() const
{
    return (ptr && status.isSet());
}
