#ifdef BORNAGAIN_PYTHON

#include "PyCore/Embed/PyCore.h"
#include "ImportMultiLayer.h"
#include "Sample/Multilayer/MultiLayer.h"
#include "PyCore/Embed/PythonInterpreter.h"  // BornAgain::importScript
// SWIG runtime access for creating a MultiLayer instance from Python
#include "auto/Wrap/swig_runtime.h"


namespace PythonInterpreter
{

Result<std::unique_ptr<MultiLayer>> createMultiLayerFromPython(
    const std::string& script, const std::string& functionName,
    const std::string& path)
{
    PyObjectPtr tmpModule {PythonInterpreter::BornAgain::importScript(script, path)};

    // locate the `get_simulation` function (it is an attribute of the module)
    PyObject* pAddFn = PyObject_GetAttrString(tmpModule.ptr, functionName.c_str());
    if (!pAddFn) {
        std::stringstream msg_ss;
        msg_ss << "PythonInterpreter::BornAgain: "
               << "Cannot locate the compiled function '" << functionName << "'";
        return {Status(Status::Type::Error, msg_ss.str())};
    }

    // create a `MultiLayer` Python object via calling the function
    PyObject* instance = PyObject_CallFunctionObjArgs(pAddFn, NULL);

    // clean up
    Py_DecRef(pAddFn);

    if (!instance) {
        std::stringstream msg_ss;
        msg_ss << "PythonInterpreter::BornAgain: "
               << "Cannot call the function '" << functionName << "'";
        return {Status(Status::Type::Error,
                       PythonInterpreter::errorDescription(msg_ss.str()))};
    }

    // Construct a C++ object from the Python object
    /* NOTE:
       This conversion requires a SWIG-produced Python API.
     */

    void* argp1 = 0;
    swig_type_info* pTypeInfo = SWIG_TypeQuery("MultiLayer *");

    const int res = SWIG_ConvertPtr(instance, &argp1, pTypeInfo, 0);
    if (!SWIG_IsOK(res)) {
        Py_DecRef(instance);
        std::stringstream msg_ss;
        msg_ss << "PythonInterpreter::BornAgain: "
               << "SWIG failed to extract a 'MultiLayer' instance "
               << "via calling the function '" << functionName << "'";
        return {Status(Status::Type::Error,
                       PythonInterpreter::errorDescription(msg_ss.str()))};
    }

    MultiLayer* multilayer = reinterpret_cast<MultiLayer*>(argp1);
    std::unique_ptr<MultiLayer> result_ptr(multilayer->clone());

    Py_DecRef(instance);

    // TODO: check if it is possible to move a unique_ptr
    return {std::move(result_ptr)};
}

}  // namespace PythonInterpreter

#endif // BORNAGAIN_PYTHON
